/*
 *  Copyright (C) 2003 Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-action-fullscreen.h"
#include "galeon-shell.h"
#include "eel-gconf-extensions.h"
#include "prefs-strings.h"
#include "gul-gobject-misc.h"
#include "gul-toggle-toolbutton.h"
#include "gul-string.h"
#include "gul-gui.h"
#include "pixbuf-cache.h"

#include <gtk/gtkstock.h>
#include <libgnome/gnome-i18n.h>
#include <string.h>

#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
#define DEBUG_MSG(x) g_print x
//#define DEBUG_MSG(x) 

static void		galeon_action_fullscreen_init			(GaleonActionFullscreen *action);
static void		galeon_action_fullscreen_class_init		(GaleonActionFullscreenClass *class);
static void		galeon_action_fullscreen_finalize		(GObject *object);
static void		galeon_action_fullscreen_connect_proxy		(EggAction *action, GtkWidget *proxy);
static GtkWidget *	galeon_action_fullscreen_create_toolitem	(EggAction *action);
static void		galeon_action_fullscreen_update			(GaleonAction *a);

static EggActionClass *parent_class = NULL;


MAKE_GET_TYPE (galeon_action_fullscreen, "GaleonActionFullscreen", GaleonActionFullscreen,
	       galeon_action_fullscreen_class_init, 
	       galeon_action_fullscreen_init, GALEON_TYPE_TOGGLE_ACTION);


static void
galeon_action_fullscreen_class_init (GaleonActionFullscreenClass *class)
{
	GObjectClass *object_class = G_OBJECT_CLASS (class);
	EggActionClass *action_class = EGG_ACTION_CLASS (class);
	GaleonActionClass *galeon_action_class = GALEON_ACTION_CLASS (class);

	object_class->finalize = galeon_action_fullscreen_finalize;

	action_class->create_tool_item = galeon_action_fullscreen_create_toolitem;
	action_class->connect_proxy = galeon_action_fullscreen_connect_proxy;

	galeon_action_class->update = galeon_action_fullscreen_update;

	parent_class = g_type_class_peek_parent (class);
}

static void
galeon_action_fullscreen_toggled_cb (GaleonToggleAction *toggle_action, GaleonActionFullscreen *action)
{
	GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (action));
	if (window && action->lock_update == 0)
	{
		galeon_window_toggle_fullscreen (window);
	}
}

static void
galeon_action_fullscreen_init (GaleonActionFullscreen *action)
{
	g_object_set (G_OBJECT (action), "stock_id", GALEON_STOCK_FULLSCREEN, NULL);
	g_object_set (G_OBJECT (action), "tooltip", _("Toggle fullscreen mode"), NULL);

	g_signal_connect (action, "toggled", 
			  G_CALLBACK (galeon_action_fullscreen_toggled_cb), action);
}

static void
galeon_action_fullscreen_finalize (GObject *object)
{
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static GtkWidget *
galeon_action_fullscreen_create_toolitem (EggAction *action)
{
	GulToggleToolbutton *ti = gul_toggle_toolbutton_new ();
	GulToolbutton *b = GUL_TOOLBUTTON (ti);

	gul_toolbutton_set_show_arrow (b, FALSE);
	gul_toolbutton_set_enable_menu (b, FALSE);
	egg_tool_item_set_homogeneous (EGG_TOOL_ITEM (b), FALSE);

	return GTK_WIDGET (ti);
}

static void
galeon_action_fullscreen_connect_proxy (EggAction *a, GtkWidget *proxy)
{
	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (a, proxy);
}

static void
galeon_action_fullscreen_update (GaleonAction *a)
{
	GaleonWindow *window = galeon_action_get_window (a);
	EmbedChromeMask mask = galeon_window_get_chrome (window);
	gboolean active = mask & EMBED_CHROME_OPENASFULLSCREEN;
	
	GALEON_ACTION_FULLSCREEN (a)->lock_update++;
	galeon_toggle_action_set_active (GALEON_TOGGLE_ACTION (a), active);
	GALEON_ACTION_FULLSCREEN (a)->lock_update--;
}
